class DataSource:
    def __init__(self):
        pass

    def get_models(self, query: str = None):
        pass

    def get_trainers(self, query: str = None):
        pass

    def get_data_loaders(self, query: str = None):
        pass

    def get_experiments(self, query: str = None):
        pass

    def get_packages(self, query: str = None):
        pass
